import importlib

from goonj.entities import SmtpServerConfig
from goonj.exception import CustomNotificationModuleNotFound


class EmailSettings(object):

    def __init__(self, email_settings):

        self.notification_service = None
        self.smtp_server_config = None

        if 'smtp_server' in email_settings:
            self.smtp_server_config = SmtpServerConfig(email_settings['smtp_server']['host'],
                                                       email_settings['smtp_server']['port'],
                                                       email_settings['smtp_server']['username'],
                                                       email_settings['smtp_server']['password'])

        if 'custom_notification_service' in email_settings:
            custom_notification_service = email_settings[
                'custom_notification_service']

            try:
                module_name, class_name = custom_notification_service.rsplit(
                    ".",
                    1)
                module = importlib.import_module(module_name)
                notification_service_class = getattr(module, class_name)
                notification_service = notification_service_class()
                self.notification_service = notification_service
            except Exception as e:
                raise CustomNotificationModuleNotFound('Custom Notifcation '
                                                       'service module not '
                                                       'found ,Please check '
                                                       'the package and '
                                                       'class name ,'
                                                       'for typos or any '
                                                       'other error')


class SmsSettings(object):

    def __init__(self, sms_settings):

        self.notification_service = None

        if 'custom_notification_service' in sms_settings:
            custom_notification_service = sms_settings[
                'custom_notification_service']

            try:
                module_name, class_name = custom_notification_service.rsplit(
                    ".",
                    1)
                module = importlib.import_module(module_name)
                notification_service_class = getattr(module, class_name)
                notification_service = notification_service_class()
                self.notification_service = notification_service
            except Exception as e:
                raise CustomNotificationModuleNotFound('Custom Notifcation '
                                                       'service module not '
                                                       'found ,Please check '
                                                       'the package and '
                                                       'class name ,'
                                                       'for typos or any '
                                                       'other error')



from goonj.entities import ErrorConfig


class ErrorConfigSetting(object):

    def __init__(self, error_code_settings):
        self.error_code_config = {}
        if error_code_settings and len(error_code_settings) > 0:
            for error_code_setting in error_code_settings:
                self.error_code_config[str(error_code_setting['error_code'])] = ErrorConfig(str(error_code_setting[
                                                                                           'error_code']),
                                                                                       error_code_setting['threshold'])





from enum import Enum

DEFAULT_CONFIG_FILE_PATH = "/var/tmp/goonj.yml"

DEFAULT_IMPORT_PATHS = '.'

DEFAULT_APP_ID = 'DefaultApp'


class Sev(Enum):
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"
    CRITICAL = "critical"


SEV_VALUES = [Sev.LOW.value, Sev.MEDIUM.value,
              Sev.HIGH.value, Sev.CRITICAL.value]

CHANNEL_SLACK = "slack_channels"
CHANNEL_SMS = "sms_channels"
CHANNEL_EMAIL = "email_channels"
CHANNEL_LOGGER = "logger_channels"

class Frequency(Enum):
    DAILY="daily"
    HOURLY="hourly"
    MONTHLY="monthly"



from goonj.conf.constants import Sev
from goonj.entities import CustomMessage


class SmartAlert(object):

    def __init__(self, name, source, logger=None,rule_engine=None):
        """

        :param name: name of the source
        :param source:  value of the source
        :param logger: logger object used for general logging
        """
        self.source = source
        self.name = name
        self.logger = logger
        self.rule_engine=rule_engine

    def warn(self, message, sev=None, subject=None, error_id=None, error=None,
             tag_list=None, *args, **kwargs):
        self.log_message('warn', message, sev, subject, error_id, error,
                         tag_list, *args, **kwargs)

    def info(self, message, sev=None, subject=None, error_id=None, error=None,
             tag_list=None, *args, **kwargs):
        self.log_and_alert_message('info', message, sev, subject, error_id, error,
                                   tag_list, *args, **kwargs)

    def error(self, message, sev=None, subject=None, error_id=None, error=None,
              tag_list=None, *args, **kwargs):
        self.log_and_alert_message('error', message, sev, subject, error_id, error,
                                   tag_list, *args, **kwargs)

    def debug(self, message, sev=None, subject=None, error_id=None, error=None,
              tag_list=None, *args, **kwargs):
        self.log_and_alert_message('debug', message, sev, subject, error_id, error,
                                   tag_list, *args, **kwargs)

    def exception(self, message, sev=None, subject=None, error_id=None, error=None,
                  tag_list=None, *args, **kwargs):
        self.log_and_alert_message('exception', message, sev, subject, error_id, error,

                                   tag_list, *args, **kwargs)

    def log_and_alert_message(self, function_name, message, sev=None, subject=None, error_id=None, error=None,
                              tag_list=None, *args, **kwargs):
        if self.logger is not None:

            if sev is None:
                sev_value = 'No Sev'
            else:
                sev_value = sev.value
            custom_message = CustomMessage(tag_list, sev_value, message, error, error_id, subject)
            getattr(self.logger, function_name)(custom_message, *args, **kwargs)
        self.__alert(sev, message, subject, error_id, error, tag_list)

    def __alert(self, sev, message, subject, error_id, error, tag_list):

        if not sev:
            return

        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type Sev')

        if self.rule_engine and not self.rule_engine.is_alerting_required(error_id):
            return


            for channel in self.source.default_channels:
                channel.send(sev, message, subject, error_id, error,
                             tag_list)

        try:
            if self.source.severity:
                severity = self.source.severity[sev.value]
                for channel in severity.channels:
                    channel.send(sev, message, subject, error_id, error,
                                 tag_list)
        except KeyError as e:
            pass
            # Can supresss this excpetion need to check behaviour
            # raise ChannelsNotDefinedForSev('Channels are not defined for {} '
            #                                'sev in config file'
            #                                ''.format(sev.value))



from goonj.conf import constants
from goonj.entities import AlertSource, Severity
from goonj.exception import EmailChannelNotDefined, SlackChannelNotDefined, SourceNotInitalized, NoSevDefined, \
    SlackChannelSettingsNotDefined, EmailChannelSettingsNotDefined, SMSChannelSettingsNotDefined, SevNotSupported


class SourceSettings(object):

    def __init__(self, sources, channels_map):
        self.alert_sources = self.get_alert_sources(sources, channels_map)

    def get_alert_sources(self, sources, channels_map):
        return {key: self.initialize_alert_source(key, value, channels_map) for
                key, value in
                sources.items()}

    def initialize_alert_source(self, key, value, channels_map):
        """

        :param key: name of the source
        :param value: value of the source
        :param channels_map:

         structure of channel_map looks like below
         email_channel=
                    {'email_channle_name1':email_channel_object1,
                     'email_channle_name2':email_channel_object2}
         sms_channel=
                    {'sms_channle_name1':sms_channel_object1,
                     'sms_channle_name2':smsl_channel_object2}


        A Source will have sev's{}
        Each sev will have channels associated to it.
        Source will also have list of default channels associate with

        channel_obj can be object of email_cahnnel_obj,sms_channel_obj etc

        source={
                'high':[channel_obj1,channel_objj2,channel_obj3],
                'low' :[channle_obj2,channle_obj4]
                'default_channels':[channel_obj5]

        }

        Channels can be of multiple type email_channel,sms_channel,
        slack_channel

        """
        if not value:
            raise SourceNotInitalized('Source is not intialized ,If no sevs,channels  are associated initialize it '
                                      'with None ')

        if value == 'None':
            return AlertSource(key, None, None)

        default_channels_obj = None
        if 'default_channels' in value:
            default_channels_obj = self.get_channels(value['default_channels'],
                                                     channels_map)

        sev_obj = self.get_sev(value['sev'], channels_map)
        return AlertSource(key, default_channels_obj, sev_obj)

    def get_channels(self, channels, channels_map):
        """

      :param channels:
            channels ={
               'email_channels':['channle_name1',channel_name2],
               'slack_channels':['channle_name3',channel_name4]
               }

      :param channels_map:

      structure of channel_map looks like below
      email_channel=
                 {'email_channle_name1':email_channel_object1,
                  'email_channle_name2':email_channel_object2}
      sms_channel=
                 {'sms_channle_name1':sms_channel_object1,
                  'sms_channle_name2':smsl_channel_object2}


      :return: list of channel objects [channel_objec1  (sms_channel),
         channle_object2  (slack_channel) ,channel_object3  (sms_channel) ]
        """

        channels_obj = []

        if not channels:
            return channels_obj

        if constants.CHANNEL_SLACK in channels:

            if not (channels_map and hasattr(channels_map, 'slack_channels')):
                raise SlackChannelSettingsNotDefined(
                    'Settings for slack channel is not defined ,please define it ')

            try:
                channels_obj.extend(
                    [channels_map.slack_channels[name] for name in
                     channels[constants.CHANNEL_SLACK]])
            except KeyError as e:
                raise SlackChannelNotDefined('Slack channel \'{}\' is used but not '
                                             'defined in configuration file'
                                             ''.format(e.args[0]))
        if constants.CHANNEL_EMAIL in channels:

            if not (channels_map and hasattr(channels_map, 'email_channels')):
                raise EmailChannelSettingsNotDefined(
                    'Settings for email channel is not defined ,please define it ')

            try:
                channels_obj.extend(
                    [channels_map.email_channels[name] for name in
                     channels[constants.CHANNEL_EMAIL]])
            except KeyError as e:
                raise EmailChannelNotDefined('Email channel  \'{}\' is used but not '
                                             'defined in configuration file'
                                             ''.format(e.args[0]))
        if constants.CHANNEL_SMS in channels:

            if not (channels_map and hasattr(channels_map, 'sms_channels')):
                raise SMSChannelSettingsNotDefined(
                    'Settings for sms channel is not defined ,please define it ')
            try:
                channels_obj.extend(
                    [channels_map.sms_channels[name] for name in
                     channels[constants.CHANNEL_SMS]])
            except KeyError as e:
                raise EmailChannelNotDefined('Sms channel  \'{}\' is used but not '
                                             'defined in configuration file'
                                             ''.format(e.args[0]))

        return channels_obj

    def get_sev(self, sev, channels_map):
        """

        :param sev: sev associated with a source
        sev={
        'high':{
               'email_channels':['channle_name1',channel_name2],
               'slack_channels':['channle_name3',channel_name4]
               },
        'low':{
               'email_channels':['channle_name1',channel_name2],
               'slack_channels':['channle_name3',channel_name4]
               }
        }
        :param
        structure of channel_map looks like below
        email_channel=
                 {'email_channle_name1':email_channel_object1,
                  'email_channle_name2':email_channel_object2}
        sms_channel=
                 {'sms_channle_name1':sms_channel_object1,
                  'sms_channle_name2':smsl_channel_object2}
        :return:

        sev={
        'high':[channel_obj1,channel_objj2,channel_obj3],
        'low' :[channle_obj2,channle_obj4]
        }



        """

        if not sev:
            raise NoSevDefined('No Sevs are defined within \'sev\' in settings file ,define at least one sev ,'
                               'or remove the sev setting')

        sev_obj = {}

        for sev_name, sev_value in sev.items():
            if sev_name in constants.SEV_VALUES:
                channels_obj = self.get_channels(sev[sev_name], channels_map)
                sev_obj[sev_name] = Severity(channels_obj)
            else:
                raise SevNotSupported(
                    'Sev {} not supported valid values are low,medium,high,critical '.format(sev_name))

        return sev_obj




class RuleEngine(object):




    def __init__(self,error_code_config):
        self.error_code_config=error_code_config
        self.error_count={}

    def is_alerting_required(self,error_code):


        if error_code in self.error_code_config.error_code_config:
            if error_code in self.error_count:
                if self.error_count[error_code] == (self.error_code_config.error_code_config[error_code].threshold)-1:
                    self.error_count[error_code]=0
                    return True
                else:
                    self.error_count[error_code]=self.error_count[error_code]+1
                    return False
            else:

                self.error_count[error_code]=1
                return False


        return True






from goonj.channels import constants
from goonj.channels.email_channel import EmailChannel
from goonj.channels.slack_channel import SlackChannel
from goonj.channels.sms_channel import SmsChannel
from goonj.exception import SMSSettingNotFound, EmailSettingNotFound, ChannelsNotDefined, EmailChannelNotDefined, \
    SlackChannelNotDefined, SMSChannelNotDefined


class ChannelSettings(object):

    def __init__(self, channels, email_settings, sms_settings):
        """
        :param channels:

        channels={
        'email_channels':[{'to':'',from_add:'','name':''},   {'to':'',from_add:'','name':''}  ],
        'slack_channels':[{'webhook:'','name':''},{'webhook:'','name':''}]
        }s

        initializes channel_map given below and add settings to the channels
        channel_map=(
        email_channel=
                 {'email_channle_name1':email_channel_object1,
                  'email_channle_name2':email_channel_object2}
        sms_channel=
                 {'sms_channle_name1':sms_channel_object1,
                  'sms_channle_name2':smsl_channel_object2} )

        """
        if not channels:
            raise ChannelsNotDefined('No channels are defined ,define at least one cahnnel within \'channels\' or '
                                     'remove chennels settign completly if no channel is needed')

        if constants.CHANNEL_EMAIL in channels:
            self.email_channels = self.get_email_channels(
                channels[constants.CHANNEL_EMAIL])

            self.register_setting_to_email_channel(email_settings)

        if constants.CHANNEL_SLACK in channels:
            self.slack_channels = self.get_slack_channel(
                channels[constants.CHANNEL_SLACK])

        if constants.CHANNEL_SMS in channels:
            self.sms_channels = self.get_sms_channel(
                channels[constants.CHANNEL_SMS])

            self.register_setting_to_sms_channel(sms_settings)

    def register_setting_to_email_channel(self, email_settings):
        try:
            # register notification service if available
            EmailChannel.register_custom_email_notifcation_service(
                email_settings.notification_service)
            # register smtp configuration
            EmailChannel.register_smtp_server_config(
                email_settings.smtp_server_config)
        except AttributeError as e:
            raise EmailSettingNotFound('For email channels ,email_settings is not defined ,please define a custom '
                                       'service/smtp '
                                       'sever setting '
                                       'to '
                                       'handle email')

    def register_setting_to_sms_channel(self, sms_settings):
        try:
            SmsChannel.register_custom_sms_notifcation_service(
                sms_settings.notification_service)
        except AttributeError as e:
            raise SMSSettingNotFound('For Sms channels, sms_settings is not defined ,please define a custom '

                                     'service to handle sms')

    def get_email_channels(self, email_channels):
        if not email_channels:
            raise EmailChannelNotDefined('No Email Channel is defined for \'email_channels\' ,please add at least '
                                         'one email channel ,or if you dont need it remove \'email_channels\'')
        return {email_channel['name']: EmailChannel(**email_channel) for
                email_channel in email_channels}

    def get_slack_channel(self, slack_channels):
        if not slack_channels:
            raise SlackChannelNotDefined('No Slack Channel is defined for \'slack_channels\' ,please add at least '
                                         'one slack channel ,or if you dont need it remove \'slack_channels\'')
        return {slack_channel['name']: SlackChannel(**slack_channel) for
                slack_channel in slack_channels}

    def get_sms_channel(self, sms_channels):
        if not sms_channels:
            raise SMSChannelNotDefined('No SMS Channel is defined for \'sms_channels\' ,please add at least '
                                       'one slack channel ,or if you dont need it remove \'sms_channels\'')
        return {sms_channel['name']: SmsChannel(**sms_channel) for
                sms_channel in sms_channels}



from goonj.conf.configuration import Configuration
from goonj.conf.constants import DEFAULT_CONFIG_FILE_PATH
from goonj.exception import GoonjNotInitalized, SourceNotDefined
from goonj.smart_alert import SmartAlert

_sources = {}
goonj = None


def get_smart_alert(name, logger=None):
    """

    :param name: name of the source
    :return: source initailized with all the sev's and associated channels

    """
    if name in _sources:
        return _sources[name]

    if goonj is None:
        raise GoonjNotInitalized

    if goonj.config.sources.alert_sources[name] is None:
        raise SourceNotDefined('source {} is not defined in configuration '
                               'file '.format(name))

    _sources[name] = goonj.config.sources.alert_sources[name]

    return SmartAlert(name, _sources[name], logger,goonj.config.rule_engine)


class Goonj(object):
    __instance = None

    def __new__(cls, **kwargs):
        if cls.__instance is None:
            cls.__instance = super(Goonj, cls).__new__(cls)
            cls.__instance.__initialized = False
        return cls.__instance

    def __init__(self, config_file_path=DEFAULT_CONFIG_FILE_PATH):
        global goonj
        if (self.__initialized):
            return
        self.__initialized = True
        self.config = Configuration(config_file=config_file_path)


        goonj = self



import abc


class BaseChannel(object):
    """
    Base class for all alerting channels
    """

    @abc.abstractmethod
    def send(self, sev, message, subject, error_id, error, tag_list):
        """

        :param sev:
        :param description:
        :param subject:
        :param error_id:
        :param error:
        :param tag_list:
        :return:
        """

    pass



import email.utils
import smtplib
import threading
from email.mime.text import MIMEText

from goonj.channels.base_channel import BaseChannel
from goonj.conf.constants import Sev
from goonj.exception import InvalidImplemnationOfNotifcationService, \
    EmailChannelNameNotDefined, EmailChannelFromAddressNotDefined, \
    EmailChannelToAddressNotDefined


class BaseCustomEmailNotificationService(object):
    def send_email_notification(self, from_add, to, sev, message, subject,
                                error_id,
                                error,
                                tag_list):
        pass


class EmailChannel(BaseChannel):
    """
    Channel to send emails
    """

    notification_service = None
    smtp_server_config = None

    def __init__(self, **kwargs):
        """

        :param kwargs:
        dict of parameters required to initialize email channel,some field
            are optional like subject
        Initializing email channel using config

        """
        if 'name' not in kwargs:
            raise EmailChannelNameNotDefined(
                'Email channel name not defined ,\'name\' attribute is required')

        if 'from_add' not in kwargs:
            raise EmailChannelFromAddressNotDefined(
                'Email channel from_add not defined ,\'from_add\' attribute '
                'is '
                'required')

        if 'to' not in kwargs:
            raise EmailChannelToAddressNotDefined('Email Channel to address '
                                                  'is not define, \'to\' is '
                                                  'required field')

        self.from_add = kwargs['from_add']
        self.to = kwargs['to']
        self.name = kwargs['name']

        if 'subject' in kwargs:
            self.subject = kwargs['subject']

    @classmethod
    def register_custom_email_notifcation_service(cls,
                                                  notification_service):
        if not isinstance(notification_service,
                          BaseCustomEmailNotificationService):
            raise InvalidImplemnationOfNotifcationService('Custom '
                                                          'Notification '
                                                          'service should '
                                                          'implement '
                                                          'BaseCustomEmailNotificationService')

        cls.notification_service = notification_service

    @classmethod
    def register_smtp_server_config(cls, smtp_server_config):
        cls.smtp_server_config = smtp_server_config

    def send_message(self, sev, message, subject=None, error_id=None,
                     error=None,
                     tag_list=None):

        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type Sev')

        if self.notification_service:
            self.notification_service.send_email_notification(self.from_add,
                                                              self.to,
                                                              sev,
                                                              message,
                                                              subject,
                                                              error_id,
                                                              error,
                                                              tag_list)

        elif self.smtp_server_config:

            msg = MIMEText(message)
            msg['To'] = email.utils.formataddr(('Recipient',
                                                self.to))
            msg['From'] = email.utils.formataddr(('Author', self.from_add))
            msg['Subject'] = subject

            server = smtplib.SMTP(self.smtp_server_config.host,
                                  self.smtp_server_config.port)

            try:

                server.sendmail(self.from_add, [self.to],
                                msg.as_string())
            except Exception as e:
                pass  # check how to handle logging in libraries
            finally:
                server.quit()

    # send email asynchronously
    def send(self, sev, message, subject=None, error_id=None,
             error=None,
             tag_list=None):
        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type Sev')

        thread = threading.Thread(target=self.send_message, args=(sev, message,
                                                                  subject,
                                                                  error_id,
                                                                  error,
                                                                  tag_list),
                                  kwargs={})
        thread.start()
        # thread.join()



import logging
import threading

import requests

from goonj.channels.base_channel import BaseChannel
# https://hooks.slack.com/services/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
from goonj.conf.constants import Sev
from goonj.exception import SlackChannelNameNotDefined
from goonj.utils import xstr

logger = logging.getLogger(__name__)


class SlackChannel(BaseChannel):

    def __init__(self, **kwargs):

        if 'name' not in kwargs:
            raise SlackChannelNameNotDefined(
                'Slack channel name not defined ,\'name\' attribute is '
                'required')

        if 'webhook' not in kwargs:
            raise SlackChannelNameNotDefined(
                'Slack channel webhook not defined ,\'webhook\' attribute is '
                'required')

        self.name = kwargs['name']
        self.webhook = kwargs['webhook']


    def send_message(self, sev, message, subject=None, error_id=None,
                     error=None,
                     tag_list=None):

        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type   Sev')

        try:
            headers = {'content-type': 'application/json'}
            final_text = "Sev: " + xstr(sev.value) + "\nsubject: " + xstr(subject) + \
                         "\ndescription: " \
                         "" + xstr(message)+"\n error_id: "+xstr(error_id)+"\nerror: "+xstr(error)+"\ntags: "+xstr(
                tag_list)

            requests.post(self.webhook, json={"text": final_text},
                          headers=headers)

        except Exception as e:
            logger.error("Error while alerting in slack(Ignoring) %s", e)

    # should it be asynchronous or synchrnoonus
    def send(self, sev, message, subject=None, error_id=None,
             error=None,
             tag_list=None):
        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type Sev')

        thread = threading.Thread(target=self.send_message, args=(sev, message,
                                                                  subject,
                                                                  error_id,
                                                                  error,
                                                                  tag_list),
                                  kwargs={})
        thread.start()
        # thread.join()  #no need to wait for notfication thread



class GoonjNotInitalized(Exception):
    pass


class SourceNotDefined(Exception):
    pass


class InvalidImplemnationOfNotifcationService(Exception):
    pass


class EmailChannelNotDefined(Exception):
    pass


class SlackChannelNotDefined(Exception):
    pass


class EmailChannelNameNotDefined(Exception):
    pass


class SlackChannelNameNotDefined(Exception):
    pass


class CustomNotificationModuleNotFound(Exception):
    pass


class ChannelsNotDefinedForSev(Exception):
    pass


class SmsChannelNameNotDefined(Exception):
    pass


class EmailChannelFromAddressNotDefined(Exception):
    pass


class EmailChannelToAddressNotDefined(Exception):
    pass


class SMSSettingNotFound(Exception):
    pass


class EmailSettingNotFound(Exception):
    pass


class SourceNotInitalized(Exception):
    pass


class ChannelsNotDefined(Exception):
    pass


class EmailChannelNotDefined(Exception):
    pass


class SlackChannelNotDefined(Exception):
    pass


class SMSChannelNotDefined(Exception):
    pass


class NoSevDefined(Exception):
    pass


class SlackChannelSettingsNotDefined(Exception):
    pass


class EmailChannelSettingsNotDefined(Exception):
    pass


class SMSChannelSettingsNotDefined(Exception):
    pass


class SevNotSupported(Exception):
    pass



import threading

from goonj.channels.base_channel import BaseChannel
from goonj.conf.constants import Sev
from goonj.exception import SmsChannelNameNotDefined, \
    InvalidImplemnationOfNotifcationService


class BaseCustomSmsNotificationService(object):
    def send_sms_notification(self, phone_numbers, sev, message, subject,
                              error_id,
                              error,
                              tag_list):
        pass


class SmsChannel(BaseChannel):
    notification_service = None

    def __init__(self, **kwargs):
        if 'name' not in kwargs:
            raise SmsChannelNameNotDefined(
                'Sms channel name not defined ,\'name\' attribute is required')

        self.phone_numbers = kwargs['phone_numbers']

    @classmethod
    def register_custom_sms_notifcation_service(self,
                                                notification_service):
        if not isinstance(notification_service,
                          BaseCustomSmsNotificationService):
            raise InvalidImplemnationOfNotifcationService('Custom '
                                                          'Notification '
                                                          'service should '
                                                          'implement '
                                                          'BaseCustomSmsNotificationService')

        self.notification_service = notification_service

    def send_message(self, sev, message, subject=None, error_id=None,
                     error=None,
                     tag_list=None):

        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type Sev')

        if self.notification_service is None:
            return  # log here that no notifcation service is registerd

        self.notification_service.send_sms_notification(self.phone_numbers,
                                                        sev,
                                                        message,
                                                        subject,
                                                        error_id,
                                                        error,
                                                        tag_list)

    # send sms asynchronously
    def send(self, sev, message, subject=None, error_id=None,
             error=None,
             tag_list=None):
        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type Sev')

        thread = threading.Thread(target=self.send_message, args=(sev, message,
                                                                  subject,
                                                                  error_id,
                                                                  error,
                                                                  tag_list),
                                  kwargs={})
        thread.start()



class AlertSource(object):
    def __init__(self, name, default_channels, severity):
        self.name = name
        self.default_channels = default_channels
        self.severity = severity


class Severity(object):
    def __init__(self, channels):
        self.channels = channels


class Tag(object):
    def __init__(self, name):
        self.name = name


class CustomMessage(object):
    __slots__ = ['tags', 'severity', 'message', 'error', 'error_id', 'subject']

    def __init__(self, tags, severity, message, error, error_id, subject):
        self.tags = tags
        self.severity = severity
        self.message = message
        self.error = error
        self.error_id = error_id
        self.subject = subject

    def __str__(self):

        return '{"subject":"%s","message":"%s","tags":"%s","severity":"%s","error":"%s","error_id":"%s"}'\
               % (self.subject,
                  self.message,
                  self.tags,
                  self.severity,

                  self.error,
                  self.error_id,

                  )


class SmtpServerConfig(object):
    def __init__(self, host, port, username, password):
        self.host = host
        self.port = port
        self.username = username
        self.password = password


class ErrorConfig(object):
    def __init__(self,error_code,threshold,frequency=None):
        self.error_code=error_code
        self.threshold=threshold
        self.frequency=frequency


import os

from setuptools import setup, find_packages


def get_version():
    basedir = os.path.dirname(__file__)
    with open(os.path.join(basedir, 'goonj/version.py')) as f:
        locals = {}
        exec(f.read(), locals)
        return locals['VERSION']
    raise RuntimeError('No version info found.')


setup(
    name='goonj',
    version=get_version(),
    packages=find_packages(exclude=['tests', 'samples']),
    install_requires=[
        'yml==0.0.1',
        'request==0.0.26',
        'PyYAML==3.12',
        'requests==2.18.4',
        'pre-commit==1.7.0',
        'pre-commit-hooks==1.2.3 '

    ],
    url='',
    license='BSD',
    author='sohit kumar,bkp',
    author_email='sohit.kumar@treebohotels.com,bibek.padhy@treebohotels.com',
    test_suite="tests",
    description='A python library to enable easy alerting services across multiple channels.')



# -*- coding: utf-8 -*-

import logging
import traceback

import yaml

from goonj.conf import constants
from goonj.conf.channel_settings import ChannelSettings
from goonj.conf.email_sms_settings import EmailSettings, SmsSettings
from goonj.conf.error_config_setting import ErrorConfigSetting
from goonj.conf.source_setting import SourceSettings
from goonj.rule_engine import RuleEngine
from goonj.utils import update_import_paths


class Configuration(object):
    """
    used to configure the service
    """

    def __init__(self, **kwargs):
        """
        :param app_name: the application specific unique name.
        :param db_type: what is the database type
        :param db_name: the name of the database
        :param db_user: user information of the database
        :param db_password: the password of the database
        :param db_node: the node of the database
        :param config_file: the path of the config file
        :param import_paths: the import path location
        """

        # first initialize everything with defaults
        self.load_with_defaults()

        # then check if config file is there if yes then load from it
        if 'config_file' in kwargs:
            self.config_file = kwargs['config_file']

        self.load_from_file(self.config_file)

        # now load any run time config change done
        self.set_config(**kwargs)

        self.db = None

    def load_with_defaults(self):
        self.app_id = constants.DEFAULT_APP_ID
        self.import_paths = constants.DEFAULT_IMPORT_PATHS
        self.config_file = constants.DEFAULT_CONFIG_FILE_PATH
        self.sms_settings = None
        self.email_settings = None
        self.channels = None
        self.error_config = {}
        self.rule_engine=None


        update_import_paths(self.import_paths)

    def set_config(self, **kwargs):
        """
        set config values
        :param kwargs: contains the dict read from config file ,with all key
        values
        :return:
        """

        if 'app_id' in kwargs:
            self.app_id = kwargs['app_id']

        if 'import_paths' in kwargs:
            self.import_paths = kwargs['import_paths']
            update_import_paths(self.import_paths)

        if 'config_file' in kwargs:
            self.config_file = kwargs['config_file']

        if 'email_settings' in kwargs:
            self.email_settings = EmailSettings(kwargs['email_settings'])

        if 'sms_settings' in kwargs:
            self.sms_settings = SmsSettings(kwargs['sms_settings'])

        if 'channels' in kwargs:
            self.channels = ChannelSettings(kwargs['channels'],
                                            self.email_settings,
                                            self.sms_settings)

        if 'sources' in kwargs:
            self.sources = SourceSettings(kwargs['sources'], self.channels)

        if 'error_config' in kwargs:
            self.error_config = ErrorConfigSetting(kwargs['error_config'])
            self.rule_engine = RuleEngine(self.error_config)





    def load_from_file(self, file_path):
        logger = logging.getLogger(self.__class__.__name__)
        try:
            with open(file_path, 'r') as stream:
                loaded_config = yaml.load(stream)
                self.set_config(**loaded_config)
        except Exception as e:
            traceback.print_exc()
            logger.error(
                "Unable to load config file: {0} with exception {1}".format(
                    str(file_path), e))
            raise e



import json
import random
import string
import sys


def get_filtered_keys(key_list, payload):
    """
    Filter the payload for the given keylist and return the filtered dict
    :param key_list: comma seperated keys in string format
    :param payload: dict payload which needs to be filtered
    :return: dict with key, value pair based on the keylist





    """

    ret_dict = dict()

    key_list = str(key_list).split(",")

    for key in key_list:
        value = payload.get(key)
        if not value:
            raise \
                KeyError("Key: {} not present in payload: {} for "
                         "keylist: {}"
                         .format(key, payload, key_list))
        ret_dict[key] = value
    return ret_dict


def get_filtered_keys_from_map(key_map, payload):
    """
    Filter the payload for the given keylist and return the filtered dict
    :param key_list: a json old_key:new_key map in string format
    :param payload: dict payload which needs to be filtered
    :return: dict with new_key, value pair based on the keylist
    """

    ret_dict = dict()
    key_map = json.loads(key_map)

    for old_key, new_key in key_map:
        value = payload.get(old_key)
        if not value:
            raise KeyError("Key: {} not present in payload: {} for key map: "
                           "{}"
                           .format(old_key, payload, key_map))
        ret_dict[new_key] = value
    return ret_dict


def update_import_paths(import_paths):
    """
    update the import paths in the system
    :param import_paths:
    :return:
    """
    if import_paths:
        sys.path = import_paths.split(':') + sys.path


def generate_unique_id(size=5):
    return ''.join(random.choice(string.digits) for _ in range(size))

def xstr(s):
    if s is None:
        return ''
    return str(s)
